package mmexample3;

import javafx.application.Application;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.stage.Stage;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.ListView;
import javafx.scene.control.SelectionMode;
import javafx.scene.control.TextArea;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Pane;
import javafx.scene.layout.VBox;

public class MartianManagerGui2ToClass extends Application {
	
	// ListView events. These might already be in your app?
	protected Button btnDisplaySelected, btnRemoveSelected, btnClosestToSelected;
	protected ListView<String> lvwMartians = new ListView<>();
	
	protected TextArea txaMessage;
	@Override
	public void start(Stage primaryStage) {
		try {
			Pane root = buildGui();
			Scene scene = new Scene(root,620,450);
			primaryStage.setTitle("Example-Select Martians");
			scene.getStylesheets().add(getClass().getResource("application.css").toExternalForm());
			primaryStage.setScene(scene);
			primaryStage.show();
		} catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	private Pane buildGui() {
		// Build components
		Pane p1 = buildSelectMartians();
		Pane p2 = buildResults();
		// Build root container
		VBox root = new VBox();
		root.getChildren().addAll(p1,p2);
		return root;
	}

	private Pane buildSelectMartians() {
		// Build ListView
		lvwMartians.getSelectionModel().setSelectionMode(SelectionMode.MULTIPLE);
		lvwMartians.setPrefHeight(150);
		lvwMartians.setPrefWidth(120);

		// Build button box.
		Label lbl = new Label("Select 1 or more Martians");
		btnDisplaySelected = new Button("Display");
		btnDisplaySelected.setOnAction(new DisplaySelectedEventHandler());
		btnRemoveSelected = new Button("Remove");
		btnRemoveSelected.setOnAction(new RemoveMartiansEventHandler());
		btnClosestToSelected =new Button("Closest to");
		btnClosestToSelected.setOnAction(new ClosestToEventHandler());
		VBox vBox = new VBox();
		vBox.getStyleClass().add("h_or_v_box");		
		vBox.getChildren().addAll(lbl, btnDisplaySelected, btnRemoveSelected, btnClosestToSelected);
		
		// Build root
		HBox hBox = new HBox();
		hBox.getStyleClass().add("h_or_v_box");			
		hBox.getStyleClass().add("boxBorder");
		hBox.getChildren().addAll(lvwMartians, vBox);
		return hBox;
	}
	


	// You already have something similar to this in your code.
	private Pane buildResults() {
		Label lbl = new Label("Results");
		txaMessage = new TextArea();
		VBox vBox = new VBox();
		vBox.getChildren().addAll(lbl, txaMessage);
		return vBox;
	}

	//------------------------------------------------------------------------------------------
	// Event Handlers
	//------------------------------------------------------------------------------------------

	/**
	 * Displays the selected martians in the text area. 
	 * 
	 * Button text: "Display", name: "btnDisplaySelected", created: buildSelectMartians()
	 */
	private class DisplaySelectedEventHandler implements EventHandler<ActionEvent> {
		@Override
		public void handle(ActionEvent event) {
			String msg = "Martians selected\n";
			msg +=       "-------------------\n";
			msg +=       "YOU WRITE THIS!\n";
			txaMessage.setText(msg);
		}
	}

	/**
	 * Removes the selected martians from the martian manager and from the listview, and
	 * displays the removed martians in the text area. Hints:
	 * (1) Obtain a list of strings from the list view, each string (e.g. G-34),
	 * corresponds to a Martian. Thus, you can split off the id, and then use it to 
	 * remove from the martian manager with its remove method.
	 * (2) The martian manager's remove returns the removed martian, which you can use 
	 * for the display in the text area.
	 * (3) The list view can return the selected items as shown in labs and class:
	 * 
	 * 		List<String> selMartiansObs = lvwMartians.getSelectionModel().getSelectedItems();
	 * 
	 * However, this is an "observable" list which means that when we use this to loop 
	 * over to do the removes from the list view itself, it will change this list which
	 * is problematic. Thus, immediately after the line above, create a new list
	 * which will be "safe" to iterate over.
	 * 
	 *      List<String> selMartians = new ArrayList<>(selMartiansObs);
	 * 
	 * (4) To remove from list view (for example): lvw.getItems().remove("G-34")
	 *   
	 * Button text: "Remove", name: "btnRemoveSelected", created: buildSelectMartians()
	 */
	private class RemoveMartiansEventHandler implements EventHandler<ActionEvent> {
		@Override
		public void handle(ActionEvent event) {
			String msg = "Martians removed\n";
			msg +=       "-------------------\n";
			msg +=       "YOU WRITE THIS!\n";
			txaMessage.setText(msg);
		}
	}

	/**
	 * Finds the martians closest to the selected martians and displays both in the text area. 
	 * 
	 * Button text: "Closest to", name: "btnClosestToSelected", created: buildSelectMartians()
	 */
	private class ClosestToEventHandler implements EventHandler<ActionEvent> {
		@Override
		public void handle(ActionEvent event) {
			String msg = "Closest To\n";
			msg +=       "-------------------\n";
			msg +=       "YOU WRITE THIS!\n";
			txaMessage.setText(msg);
		}
	}
	
	public static void main(String[] args) {
		launch(args);
	}
}
